/// <reference types="codemirror" />
declare global {
    namespace HyperMD {
        interface HelperCollection extends Object {
        }
        interface EditorConfiguration {
        }
        interface Editor {
            hmd: HelperCollection;
        }
    }
}
export declare type cm_t = CodeMirror.Editor;
declare module "codemirror" {
    /** initial value for options */
    const Init: any;
    var hyperMDModes: {
        [mode: string]: CodeMirror.Mode<any>;
    };
    /**
     * Compare two positions, return 0 if they are the same,
     * a negative number when a is less, and a positive number otherwise.
     */
    function cmpPos(a: CodeMirror.Position, b: CodeMirror.Position): number;
    function addClass(el: HTMLElement, className: string): any;
    function rmClass(el: HTMLElement, className: string): any;
    function contains(parent: HTMLElement, child: Node): boolean;
    function defineMIME(mime: string, mode: string): any;
    function defineMode<T>(id: string, modefactory: CodeMirror.ModeFactory<T>, alias: string | string[]): void;
    function startState<T>(mode: CodeMirror.Mode<T>): T;
    function copyState<T>(mode: CodeMirror.Mode<T>, state: T): T;
    interface Mode<T> {
        innerMode?<U>(state: T): {
            state: U;
            mode: CodeMirror.Mode<U>;
        };
    }
    /** Multi-stroke key bindings can be specified by
     * separating the key names by spaces in the property name,
     * for example Ctrl-X Ctrl-V.
     *
     * When a map contains multi-stoke bindings or keys with modifiers that
     * are not specified in the default order (Shift-Cmd-Ctrl-Alt) */
    type HyperMDKeyMap = {
        [keyName: string]: Command | ((cm: CodeMirror.Editor) => void) | string;
    };
    type BuiltinCommand = "selectAll" | "singleSelection" | "killLine" | "deleteLine" | "delLineLeft" | "delWrappedLineLeft" | "delWrappedLineRight" | "undo" | "redo" | "undoSelection" | "redoSelection" | "goDocStart" | "goDocEnd" | "goLineStart" | "goLineStartSmart" | "goLineEnd" | "goLineRight" | "goLineLeft" | "goLineLeftSmart" | "goLineUp" | "goLineDown" | "goPageUp" | "goPageDown" | "goCharLeft" | "goCharRight" | "goColumnLeft" | "goColumnRight" | "goWordLeft" | "goWordRight" | "goGroupLeft" | "goGroupRight" | "delCharBefore" | "delCharAfter" | "delWordBefore" | "delWordAfter" | "delGroupBefore" | "delGroupAfter" | "indentAuto" | "indentMore" | "indentLess" | "insertTab" | "insertSoftTab" | "defaultTab" | "transposeChars" | "newlineAndIndent" | "toggleOverwrite" | "save" | "find" | "findNext" | "findPrev" | "replace" | "replaceAll" | "newlineAndIndentContinueMarkdownList";
    type Command = keyof CommandFunctions;
    var keyMap: {
        [keymapName: string]: HyperMDKeyMap;
    };
    var hyperMDCommands: CommandFunctions;
    interface CommandFunctions extends Record<BuiltinCommand, (cm: cm_t) => any> {
        hmdNewline: (cm: cm_t) => any;
        hmdNewlineAndContinue: (cm: cm_t) => any;
        hmdShiftTab: (cm: cm_t) => any;
        hmdTab: (cm: cm_t) => any;
    }
    function normalizeHyperMDKeyMap(keymap: HyperMDKeyMap): object;
    interface ModeMeta {
        name: string;
        mime: string;
        mode: string;
        ext?: string[];
    }
    function findModeByName(lang: string): ModeMeta;
    interface EditorConfiguration extends HyperMD.EditorConfiguration {
        autoCloseBrackets?: AutoCloseBrackets | boolean | string;
    }
    interface StringStream {
        lineOracle: any;
        lookAhead(lineCount: number): string;
    }
    interface LineWidget {
        on(event: "redraw", fn: Function): void;
        off(event: "redraw", fn: Function): void;
    }
    interface Editor extends CodeMirror.Doc, HyperMD.Editor {
        display: any;
        options: any;
        getAllMarks(): CodeMirror.TextMarker[];
        /**
         * This is a (much) cheaper version of getTokenAt useful for when you just need the type of the token at a given position,
         * and no other information.
         *
         * Will return null for unstyled tokens, and a string, potentially containing multiple space-separated style names, otherwise.
         */
        getTokenTypeAt(pos: CodeMirror.Position): string;
        execCommand(cmd: Command): void;
        listSelections(): {
            anchor: CodeMirror.Position;
            head: CodeMirror.Position;
            empty(): boolean;
        }[];
        replaceSelections(replacements: string[], select?: "around" | "start"): void;
        startOperation(): void;
        endOperation(): void;
    }
    interface TextMarker {
        changed(): void;
        className: string;
        /**
         * Fired when the cursor enters the marked range.
         * From this event handler, the editor state may be inspected but not modified,
         * with the exception that the range on which the event fires may be cleared.
         */
        on(eventName: "beforeCursorEnter", handler: (this: CodeMirror.Editor) => void): void;
        off(eventName: "beforeCursorEnter", handler: (this: CodeMirror.Editor) => void): void;
        /**
         * Fired when the range is cleared, either through cursor movement in combination
         * with `clearOnEnter` or through a call to its clear() method.
         *
         * Will only be fired once per handle.
         *
         * Note that deleting the range through text editing does not fire this event,
         * because an undo action might bring the range back into existence.
         *
         * `from` and `to` give the part of the document that the range spanned when it was cleared.
         */
        on(eventName: "clear", handler: (this: CodeMirror.Editor, from: CodeMirror.Position, to: CodeMirror.Position) => void): void;
        off(eventName: "clear", handler: (this: CodeMirror.Editor, from: CodeMirror.Position, to: CodeMirror.Position) => void): void;
        on(eventName: "hide" | "unhide", handler: Function): void;
        off(eventName: "hide" | "unhide", handler: Function): void;
    }
    interface LineHandle {
        styles?: (string | number)[];
        parent: any;
        lineNo(): number;
        markedSpans?: {
            from: number | null;
            to: number | null;
            marker: CodeMirror.TextMarker;
        }[];
    }
    /** CodeMirror internal Object */
    interface LineView {
        line: CodeMirror.LineHandle;
        lineN: number;
        rest: CodeMirror.LineHandle[];
        hidden?: boolean;
        text: HTMLPreElement;
        measure?: {
            heights?: number[];
            cache?: object;
            map?: (number | Text | HTMLSpanElement)[];
            caches?: object[];
            maps?: (number | Text | HTMLSpanElement)[][];
        };
    }
}
